/**
 ******************************************************************************
 * @project  interrupt example 
 * @author   Edward Ahn, Jim Jun
 * @version  V1.0.1
 * @date     26-MAY-2016
 * @brief    
*******************************************************************************
**/

/* 인터럽트 설명
  아두이노 인터럽트는 특정 핀의 입력 상태가 바뀔때 아두이노가 이를 자동으로 감지해서
  모든 동작을 잠시 멈춘 다음, ISR(Interrupt Service Routines) 이라 불리는 미리 지정된 
  함수를 실행하고, 다시 원래 작업으로 복귀합니다.
*/

/* 주의사항
  1. 인터럽트 callback 함수(ISR, Interrupt Service Routines)는 파라미터를 전달하거나 리턴할 수 없습니다. void xxx_callback() 형태가 됩니다.
  2. ISR 안에서는 delay() 함수를 사용할 수 없습니다.
  3. 그리고 milli-단위의 시각을 가져오는 함수인 millis() 를 사용하더라도 값이 증가하지 않습니다. delayMicroseconds() 의 경우에는 인터럽트에 독립적이므로 정상 동작합니다.
  4. ISR 안에서 Serial data를 읽을 경우 값이 소실됩니다.
  5. ISR 안에서 수정되는 전역 변수는 volatile 로 선언되어야 합니다.
  6. ISR 은 최대한 짧고 빠르게 수행되도록 작성해야 합니다.  
  7. 여러개의 ISR이 등록되어 있더라도 동시에 수행되지는 않습니다.
  8. 블루투스 모듈과의 통신을 위해 주로 사용되는 SoftwareSerial 라이브러리의 경우 내부적으로 인터럽트를 
     사용하는 것으로 알려져 있습니다. 따라서 UNO 보드의 경우 D2, D3 에 연결해야만 정상동작 합니다. 
     이를 회피하기 위해서 D0, D1 핀에 연결해서 Hardware Serial 로 동작시킬 수 있습니다.
*/

/* 인터럽트 등록 함수

  void attachInterrupt(interrupt, ISR, mode)
  
  interrupt: 인터럽트 번호, 핀 번호 아님! 아두이노 DUE 보드만 핀 번호로 사용. (int)
  ISR: 인터럽트가 발생할 때 수행할 callback 함수
  mode: 인터럽트가 수행될 조건
  LOW: pin 이 LOW 상태일 때
  CHANGE: pin 입력 값이 변경 될 때
  RISING: LOW -> HIGH 로 변경 될 때
  FALLING: HIGH -> LOW 로 변경 될 때
  (HIGH: HIGH 상태일 때 – 아두이노 DUE 에서만 지원)
*/

/* 아두이노 인터럽트 번호
  Arduino Mega의 경우 0 ~ 5번까지 번호 사용 가능
  Arduino Uno는 0, 1번만 사용 가능
  WizArduin는 2번을 제외하고 all pins 사용 가능

  인터럽트 번호   0    1   2   3   4    5
  Mega 2560     D2  D3  D21  D20 D19  D18
  Uno           D2  D3
  Leonardo      D3  D2  D0   D1  D7
  WizArduino    all pins (except D2)
  
*/
const int LED = 13; // LED 선언
const int SW = 3; // Interrupt할 SW 선언
volatile int state = LOW;
 
void setup()
{
  pinMode(LED, OUTPUT); // LED Output 설정
  // Interrupt 함수 등록 , (0번 인터럽트, 인터럽트 발생할 ISR, LOW <-> HIGH  변경될 때)
  pinMode(SW, INPUT); // SW Input 설정
  attachInterrupt(SW, interrupt, CHANGE);

  // Open serial communications and wait for port to open:
  //SerialUSB.begin(9600);
  //while (!SerialUSB) {
  //  ; // wait for serial port to connect. Needed for Leonardo only
  //}
}
 
void loop()
{
  digitalWrite(LED, state);  // 평소에는 LED OFF, 인터럽트 시 LED ON
  //SerialUSB.println(state);
}
 
void interrupt() // 인터럽트 함수
{
  state = !state;
  //SerialUSB.println("Interrupt");
}
